# Ejemplo

Está app muestra el uso del widget consumiendo servicios desde [JSONPlaceholder](https://jsonplaceholder.typicode.com/).

## Consumiendo servicios REST
~~~dart
import 'package:http/http.dart' as http;
class Routes {
	static const String url = 'https://jsonplaceholder.typicode.com/';

	static Future<List<dynamic>> fetch(String service) async {
		var result;
		await http.get('$url$service').then((r) {
			if (r.statusCode == 200) result = json.decode(r.body);
		}).catchError((e, s) => print(e));
		return result;
	}
	...
	static Future<List<User>> users() async {
		var list;
		await fetch('users')
			.then((l) => list = l.map((json) => User.fromJson(json)).toList());
		return list;
	}
	...
~~~
## Objeto _User_
Les recomiendo una página útil para crear objetos `Dart` a partir de una representación `JSON`, [JSON to Dart](https://javiercbk.github.io/json_to_dart/)

~~~dart
class User {
	...
	User.fromJson(Map<String, dynamic> json) {
		id = json['id'];
		name = json['name'];
		username = json['username'];
		email = json['email'];
		address =
		    json['address'] != null ? new Address.fromJson(json['address']) : null;
		phone = json['phone'];
		website = json['website'];
		company =
		    json['company'] != null ? new Company.fromJson(json['company']) : null;
	}
	...
}
~~~
## Interfaz gráfica
~~~dart
User _user;
Future<List<User>> _find(String text) async => await Routes.users();
...
GenericFormField<User>(
	minimumLenghtRequired: 0,
	dialogTitle: 'User',
	decoration: InputDecoration(labelText: 'User'),
	searchCallback: _find,
	initialValue: _user,
	validator: (u) => u == null ? 'Must select a user' : null,
	onSaved: (u) => setState(() => _user = u),
),
~~~
![screenshot](https://github.com/johnmoscoso911/generic_form_field/raw/master/images/Screenshot02.png)
## Resultado
![screenshot](https://github.com/johnmoscoso911/generic_form_field/raw/master/images/Screenshot04.png)
![screenshot](https://github.com/johnmoscoso911/generic_form_field/raw/master/images/Screenshot05.png)
### Navegación
![screenshot](https://github.com/johnmoscoso911/generic_form_field/raw/master/images/Screenshot06.png)
![screenshot](https://github.com/johnmoscoso911/generic_form_field/raw/master/images/Screenshot07.png)